#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "file.h"
#include "lzss.h"
#include "types.h"

int main()
{
    byte *window, *read_ahead, flags, bits;
    int c, d, i, j, length, start, result;
    file_t rom, decomp, comp;
	flag_t flag_pos;


	strcpy(comp.filename, "compressed.bin");
	comp.size = 0;
	comp.ptr = NULL;

	strcpy(rom.filename, "vgundam.smc");
	strcpy(decomp.filename, "title.bin");

	load_file(&decomp);
	if(!decomp.ptr)
	{
	 printf("A problem occurred.\n");
	 system("pause");
	 return 1;
	}

	load_file(&rom);
	if(!rom.ptr)
	{
	 printf("A problem occurred.\n");
	 system("pause");
	 return 1;
	}

	read_ahead = (byte *)malloc(MAX_MATCH);

	window = (byte *)malloc(WIN_SIZE);
	if(!window)
	{
	 printf("Failed to allocate memory for the window.\n");
	 system("pause");
	 return 1;
	}

	memset(window, 0, WIN_SIZE);
	
	d = 0;
	c = 0;
	window[c++] = (decomp.size & 0xff); //window[0] - [1] = length of decompressed data
	window[c++] = (decomp.size & 0xff00) >> 8;

	while(d < decomp.size) //compress all of the data
	{
	 if(c >= 0xfef) //clear window
	 {
	  comp.ptr = (unsigned char *)realloc(comp.ptr, comp.size + c);

	  if(!comp.ptr)
	  {
	   printf("An error occured while allocating memory for the compressed data.\n");
	   return 1;
	  }

	  memmove(comp.ptr + comp.size, window, c);

	  comp.size += c;
	  memset(window, 0, WIN_SIZE);
	  c = 0;
	 }

	 flag_pos.comp = c++; //preserve indexes
	 flag_pos.decomp = d;
	 bits = 0;

	 while(bits < 8)
	 {
	  j = d;

	  memset(read_ahead, 0, MAX_MATCH); //clear the read ahead buffer

 	  for(i = 0; (i <= MAX_MATCH) && (j < sizeof(decomp)); i++)
	  {
	   read_ahead[i] = decomp.ptr[j++];
	  }

	  result = scan_window(window, read_ahead, c);

	  if(result == 0) //uncompressed - 1 byte
	  {
	   window[flag_pos.comp] |= (1 << bits);
	   bits++;
	   window[c++] = decomp.ptr[d++];
	  }
	  else if(result == -1)
	  {
	   printf("An error occured while scanning the window.\n");
	   system("pause");
	   return 1;
	  }
	  else if(result == CLR_WIN) //Window filled - unused
	  {
	   c = flag_pos.comp; //start over at the bits ctrl code after clearing the window

	   comp.ptr = (unsigned char *)realloc(comp.ptr, comp.size + c);
	   memmove(comp.ptr + comp.size, window, c);

	   comp.size += c;
	   memset(window, 0, WIN_SIZE);

	   c = 0;
	   d = flag_pos.decomp;
	   bits = 8;
	  }
	  else //compressed - 2 bytes
	  {
	   window[flag_pos.comp] |= (0 << bits);
	   bits++;

	   window[c++] = (result & 0xff);
	   window[c++] = ((result & 0xff00) >> 8);
	   d += ((result & 0xf) + 2);
	  }
	 } //while(bits < 8)
	} //while(c < decomp.size)

//    comp.ptr = (unsigned char *)realloc(comp.ptr, comp.size + c);
//	memmove(comp.ptr + comp.size, window, c);

//	comp.size += c;

	write_file(comp);

	printf("Uncompressed: $%X\tCompressed: $%X\n", decomp.size, comp.size);

	free(window);
	free(rom.ptr);
	free(decomp.ptr);

	printf("File compressed!\n");

	system("pause");
	return 0;
}